#include <jtest/UnitLogger.hpp>

namespace jtest
{
    UnitLogger::UnitLogger(const std::string& name) : ConsoleLogger(name) {
        this->timestampColor = Colors::Gray;
        this->contextColor = Colors::White;
        this->locationColor = Colors::Gray;
        this->pointerColor = Colors::White;
        this->messageColor = Colors::Gray;
        this->failColor = Colors::Red;
        this->passColor = Colors::Green;
        this->healthGoodColor = Colors::Green;
        this->healthWarningColor = Colors::Yellow;
        this->healthCriticalColor = Colors::Red;
#ifdef WIN32
        IncludeColor = false;
#endif
    }

    void UnitLogger::Log(const std::string message) {
        //LogTimestamp();
        os << jlog::token(context, "[]", contextColor).Stringer(IncludeColor);
        os << jlog::token{"*", "", pointerColor}.Stringer(IncludeColor)
           << jlog::token(message, "", messageColor).Stringer(IncludeColor) << "\n";
    }

    void UnitLogger::LogTimestamp(jlog::Timestamp ts) {
        os << jlog::token{std::format(
                "{}-{}-{} {}:{}:{}.{}",
                ts.Year(),
                ts.Month(),
                ts.Day(),
                ts.Hour().count(),
                ts.Minute().count(),
                ts.Second().count(),
                ts.Millisecond().count()), "[]", timestampColor}.Stringer(IncludeColor);
    }

    void UnitLogger::LogTestResult(Test& t, bool passed, int index, int total) {
        Color4 stateColor;
        std::string state;

        if (passed) {
            stateColor = passColor;
            state = "PASSED";
        } else {
            stateColor = failColor;
            state = "FAILED";
        }

        auto l = t.Location();

        const auto tally = jlog::token(std::format("{}/{}", index, total), "[]", stateColor);
        const auto status = jlog::token(state, "[]", stateColor);
        const auto testname = jlog::token(std::format("{}::{}", context, t.Name()), "[]", stateColor).Stringer(IncludeColor);
        const auto source = jlog::token{std::format("{}:{}", l.file_name(), l.line()), "[]", locationColor}.Stringer(IncludeColor);

        os << tally
           << status
           << testname
           << source
           << "\n";
    }
    void UnitLogger::LogUnitResult(int tests_total, int tests_ran, int tests_passed, int tests_failed) {
        float tests_ran_percentage = (float)tests_ran / (float)tests_total;
        float tests_passed_percentage = (float)tests_passed / (float)tests_ran;
        float tests_failed_percentage = (float)tests_failed / (float)tests_ran;

        Color4 stateColor;
        std::string state;


        float health = tests_passed_percentage * 100;

        // health levels
        // 100 - PERFECT
        // 99  - EXCELLENT
        // 95  - GOOD
        // 75  - WARNING
        // 50  - CRITICAL
        // 25  - DISASTER


        if (health >= 100.f)
        {
            stateColor = healthGoodColor;
            state = "PERFECT";
        } else if (health >= 99.f ) {
            stateColor = healthGoodColor;
            state = "EXCELLENT";
        } else if (health >= 95.f ) {
            stateColor = healthGoodColor;
            state = "GOOD";
        } else if (health >= 75.f) {
            stateColor = healthWarningColor;
            state = "OKAY";
        } else if (health >= 50.f) {
            stateColor = healthWarningColor;
            state = "WARNING";
        } else if (health >= 25.f) {
            stateColor = healthCriticalColor;
            state = "CRITICAL";
        } else {
            stateColor = healthCriticalColor;
            state = "DISASTER";
        }

        // UnitName: X tests ran of Y, W passed, Z failed. Health Score: W / X;

        os << jlog::token(context, "[]", stateColor).Stringer(IncludeColor);
        os << std::format("Results: {}/{} tests ran, {}/{} passed, {}/{} failed.", tests_ran, tests_total, tests_passed, tests_ran, tests_failed, tests_ran);
        os << std::format(" Report Card: {:.0f}% ", health);
        os << jlog::token(state, "[]", stateColor);
        os << "\n";



    };
}